﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace erettsegi17_Rejtveny
{
    class Hajó
    {
        public int Kx { get; set; }
        public int Ky { get; set; }

        public Hajó(int x, int y)
        {
            Kx = x;
            Ky = y;
        }

        public double Táv(Hajó h2) { return Math.Sqrt(Math.Pow(this.Kx - h2.Kx, 2) + Math.Pow(this.Ky - h2.Ky, 2)); }
    }


    class Torony:Hajó //A Torony osztály a Hajó osztáy leszármazottja, a Hajó oszály a Torony osztály őse
    {
        public int Db { get; set; }

        public Torony(int x, int y, int db)
            : base (x,y) //ős konstruktorának hívása
        {
            Db = db;
        }

        public string Minősít(){return Db > 3 ? "Nehéz torony" : "";}

        
        //Halmazok összehasonlításához kell a 3. feladatban (SetEquals-hoz):
        public override bool Equals(object obj) //mikor tekíntünk két példányt egyenlőnek
        {
            Torony t = (Torony)obj;
            return this.Kx == t.Kx && this.Ky == t.Ky && this.Db==t.Db;
        }

        //Halmazok összehasonlításához kell a 3. feladatban (SetEquals-hoz):
        public override int GetHashCode() //Egyedi kódot rendel a példányokhoz
        {
            return Kx * 1000 + Ky * 10 + Db;
        }

    }

    class Feladat
    {
        public HashSet<Torony> Tornyok { get; set; }

        public Feladat()
        {
            Tornyok = new HashSet<Torony>();
        }
    }

    class Megoldás:Feladat
    {
        public string Név { get; set; }
        public List<Hajó> Hajók { get; set; }

        public Megoldás():base()
        {
            Hajók = new List<Hajó>();
        }

        public bool JóDarab { get { return Hajók.Count() == 12; } }

        public bool JóSzomszéd
        {
            get
            {
                bool jó = true;
                foreach (var vHajó in Hajók)
                {
                    foreach (var aktHajó in Hajók)
                        if (vHajó.Táv(aktHajó) != 0 && vHajó.Táv(aktHajó) < 2) jó = false;
                    foreach (var aktTorony in Tornyok)
                        if (vHajó.Táv(aktTorony) < 2) jó = false;
                }
                return jó;
            }
        }

        public bool JóTorony
        {
            get
            {
                bool jóTorony = true;
                foreach (var aktTorony in Tornyok)
                {
                    if (aktTorony.Db != //toronyból látható hajók
                        Hajók.Where(h => h.Kx == aktTorony.Kx).Count() + //hajók száma függőlegesen
                        Hajók.Where(h => h.Ky == aktTorony.Ky).Count()) //hajók száma vízszintesen
                        jóTorony = false;
                }
                return jóTorony;
            }
        }
    }

    class Rejtvény
    {
        static void Main()
        {
            Console.Write("1. Feladat: Kérek a vizsgált torony jellemzőit (x,y,hdb): ");
            string[] input = Console.ReadLine().Split(',');
            Torony vTorony = new Torony(int.Parse(input[0]),int.Parse(input[1]),int.Parse(input[2])); //Vizsgált torony
            Console.WriteLine(vTorony.Minősít());

            Console.WriteLine("2. Feladat: Tiltott helyek listája:");
            for (int x = 1; x <= 10; x++)
                for (int y = 1; y <= 10; y++)
                {
                    Hajó aktHajó = new Hajó(x, y); 
                    double táv = vTorony.Táv(aktHajó); //vizsgált torony és az aktuális hajó távolsága
                    if (táv<2 && táv!=0) Console.WriteLine("\t{0},{1}",x,y);
                }

            Console.WriteLine("3. Feladat: Nem a feladatra küldtek megoldást:");

            //Beolvas, példányosít megoldások
            List<Megoldás> megoldások = new List<Megoldás>();
            using (StreamReader sr = new StreamReader("megoldas.txt"))
            {
                sr.ReadLine();//darabszám, felesleges információ
                while (!sr.EndOfStream)
                {
                    Megoldás m = new Megoldás();
                    m.Név = sr.ReadLine();//kinek a megoldása
                    for (int sor = 1; sor <= 10; sor++) //10db sor beolvasása
                    {
                        string[] s = sr.ReadLine().Split(' ');//sort olvas, split
                        for (int oszlop = 0; oszlop < s.Length; oszlop++)
                        {
                            if (s[oszlop] == "0") continue; //ha víz
                            if (s[oszlop] == "11") { m.Hajók.Add(new Hajó(sor, oszlop+1)); continue; } //ha hajó
                            m.Tornyok.Add(new Torony(sor, oszlop+1, int.Parse(s[oszlop])));//ha nem víz és nem hajó, akkor torony
                        }
                    }
                    megoldások.Add(m);
                }
            }


            //Beolvas, példányosít feladvány
            Feladat feladat = new Feladat(); //Feladat tárolása
            using (StreamReader sr = new StreamReader("feladvany.txt"))
            {
                for (int sor = 1; sor < 11; sor++)
                {
                    string[] s = sr.ReadLine().Split(' ');
                    for (int oszlop = 0; oszlop < s.Length; oszlop++)
                    {
                        if (s[oszlop] == "0") continue;//ha víz
                        //ha nem víz, akkor torony:
                        feladat.Tornyok.Add(new Torony(sor, oszlop + 1, int.Parse(s[oszlop])));
                    }
                }

            }

            //a feladra érkezett megoldások "megoldások2":
            var megoldások2 = megoldások.Where(x => x.Tornyok.SetEquals(feladat.Tornyok)); //Ha A=B, akkor A.SetEquals(B)->true
            if (megoldások2.Count() == megoldások.Count()) Console.WriteLine("Mindegyik megoldás erre a heti feladványra érkezett!");
            else
            {   //nem a feladatra érkezett megolásokhoz tartozó nevek:
                foreach (var i in megoldások)  if (!megoldások2.Contains(i)) Console.WriteLine("\t{0}",i.Név);
            }

            Console.WriteLine("4. Feladat: Nem 12db hajó van {0} esetben.",
                megoldások2.Where(x => !x.JóDarab).Count());
          
            Console.WriteLine("5. Feladat: Rossz a szomszédsági szabály {0} esetben.",
                megoldások2.Where(x => x.JóDarab && !x.JóSzomszéd).Count());
            
            Console.WriteLine("6. Feladat: Jó megoldások:");
            foreach (var i in megoldások2.Where(x => x.JóDarab && x.JóSzomszéd && x.JóTorony))
                Console.WriteLine("\t{0}",i.Név);

            Console.ReadKey();
        }
    }
}